# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import currency_list_params, currency_create_params, currency_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.currency_response import CurrencyResponse

__all__ = ["CurrenciesResource", "AsyncCurrenciesResource"]


class CurrenciesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CurrenciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CurrenciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CurrenciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return CurrenciesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        max_decimal_places: int | NotGiven = NOT_GIVEN,
        rounding_mode: Literal["UP", "DOWN", "CEILING", "FLOOR", "HALF_UP", "HALF_DOWN", "HALF_EVEN", "UNNECESSARY"]
        | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CurrencyResponse:
        """
        Creates a new Currency for the specified Organization.

        Used to create a Currency that your Organization will start to use.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          max_decimal_places: Indicates the maximum number of decimal places to use for this Currency.

          rounding_mode

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/picklists/currency",
            body=maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "max_decimal_places": max_decimal_places,
                    "rounding_mode": rounding_mode,
                    "version": version,
                },
                currency_create_params.CurrencyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CurrencyResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CurrencyResponse:
        """Retrieve the specified Currency with the given UUID.

        Used to obtain the details
        of a specified existing Currency in your Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/picklists/currency/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CurrencyResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        max_decimal_places: int | NotGiven = NOT_GIVEN,
        rounding_mode: Literal["UP", "DOWN", "CEILING", "FLOOR", "HALF_UP", "HALF_DOWN", "HALF_EVEN", "UNNECESSARY"]
        | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CurrencyResponse:
        """
        Update a Currency with the given UUID.

        Used to update the attributes of the specified Currency for the specified
        Organization.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          max_decimal_places: Indicates the maximum number of decimal places to use for this Currency.

          rounding_mode

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/picklists/currency/{id}",
            body=maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "max_decimal_places": max_decimal_places,
                    "rounding_mode": rounding_mode,
                    "version": version,
                },
                currency_update_params.CurrencyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CurrencyResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        archived: bool | NotGiven = NOT_GIVEN,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[CurrencyResponse]:
        """
        Retrieve a list of Currencies.

        Retrieves a list of Currencies for the specified Organization. This endpoint
        supports pagination and includes various query parameters to filter the
        Currencies based on Currency ID, and short codes.

        Args:
          archived: Filter by archived flag. A True / False flag indicating whether to return
              Currencies that are archived _(obsolete)_.

              - TRUE - return archived Currencies.
              - FALSE - archived Currencies are not returned.

          codes: An optional parameter to retrieve specific Currencies based on their short
              codes.

          ids: An optional parameter to filter the list based on specific Currency unique
              identifiers (UUIDs).

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Currencies in a paginated list.

          page_size: Specifies the maximum number of Currencies to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/picklists/currency",
            page=SyncCursor[CurrencyResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    currency_list_params.CurrencyListParams,
                ),
            ),
            model=CurrencyResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CurrencyResponse:
        """
        Delete the Currency with the given UUID.

        Used to remove an existing Currency from your Organization that is no longer
        required.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/picklists/currency/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CurrencyResponse,
        )


class AsyncCurrenciesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCurrenciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCurrenciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCurrenciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncCurrenciesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        max_decimal_places: int | NotGiven = NOT_GIVEN,
        rounding_mode: Literal["UP", "DOWN", "CEILING", "FLOOR", "HALF_UP", "HALF_DOWN", "HALF_EVEN", "UNNECESSARY"]
        | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CurrencyResponse:
        """
        Creates a new Currency for the specified Organization.

        Used to create a Currency that your Organization will start to use.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          max_decimal_places: Indicates the maximum number of decimal places to use for this Currency.

          rounding_mode

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/picklists/currency",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "max_decimal_places": max_decimal_places,
                    "rounding_mode": rounding_mode,
                    "version": version,
                },
                currency_create_params.CurrencyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CurrencyResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CurrencyResponse:
        """Retrieve the specified Currency with the given UUID.

        Used to obtain the details
        of a specified existing Currency in your Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/picklists/currency/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CurrencyResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        max_decimal_places: int | NotGiven = NOT_GIVEN,
        rounding_mode: Literal["UP", "DOWN", "CEILING", "FLOOR", "HALF_UP", "HALF_DOWN", "HALF_EVEN", "UNNECESSARY"]
        | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CurrencyResponse:
        """
        Update a Currency with the given UUID.

        Used to update the attributes of the specified Currency for the specified
        Organization.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          max_decimal_places: Indicates the maximum number of decimal places to use for this Currency.

          rounding_mode

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/picklists/currency/{id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "max_decimal_places": max_decimal_places,
                    "rounding_mode": rounding_mode,
                    "version": version,
                },
                currency_update_params.CurrencyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CurrencyResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        archived: bool | NotGiven = NOT_GIVEN,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[CurrencyResponse, AsyncCursor[CurrencyResponse]]:
        """
        Retrieve a list of Currencies.

        Retrieves a list of Currencies for the specified Organization. This endpoint
        supports pagination and includes various query parameters to filter the
        Currencies based on Currency ID, and short codes.

        Args:
          archived: Filter by archived flag. A True / False flag indicating whether to return
              Currencies that are archived _(obsolete)_.

              - TRUE - return archived Currencies.
              - FALSE - archived Currencies are not returned.

          codes: An optional parameter to retrieve specific Currencies based on their short
              codes.

          ids: An optional parameter to filter the list based on specific Currency unique
              identifiers (UUIDs).

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Currencies in a paginated list.

          page_size: Specifies the maximum number of Currencies to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/picklists/currency",
            page=AsyncCursor[CurrencyResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    currency_list_params.CurrencyListParams,
                ),
            ),
            model=CurrencyResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CurrencyResponse:
        """
        Delete the Currency with the given UUID.

        Used to remove an existing Currency from your Organization that is no longer
        required.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/picklists/currency/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CurrencyResponse,
        )


class CurrenciesResourceWithRawResponse:
    def __init__(self, currencies: CurrenciesResource) -> None:
        self._currencies = currencies

        self.create = to_raw_response_wrapper(
            currencies.create,
        )
        self.retrieve = to_raw_response_wrapper(
            currencies.retrieve,
        )
        self.update = to_raw_response_wrapper(
            currencies.update,
        )
        self.list = to_raw_response_wrapper(
            currencies.list,
        )
        self.delete = to_raw_response_wrapper(
            currencies.delete,
        )


class AsyncCurrenciesResourceWithRawResponse:
    def __init__(self, currencies: AsyncCurrenciesResource) -> None:
        self._currencies = currencies

        self.create = async_to_raw_response_wrapper(
            currencies.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            currencies.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            currencies.update,
        )
        self.list = async_to_raw_response_wrapper(
            currencies.list,
        )
        self.delete = async_to_raw_response_wrapper(
            currencies.delete,
        )


class CurrenciesResourceWithStreamingResponse:
    def __init__(self, currencies: CurrenciesResource) -> None:
        self._currencies = currencies

        self.create = to_streamed_response_wrapper(
            currencies.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            currencies.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            currencies.update,
        )
        self.list = to_streamed_response_wrapper(
            currencies.list,
        )
        self.delete = to_streamed_response_wrapper(
            currencies.delete,
        )


class AsyncCurrenciesResourceWithStreamingResponse:
    def __init__(self, currencies: AsyncCurrenciesResource) -> None:
        self._currencies = currencies

        self.create = async_to_streamed_response_wrapper(
            currencies.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            currencies.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            currencies.update,
        )
        self.list = async_to_streamed_response_wrapper(
            currencies.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            currencies.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import datetime
from typing import Union
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CommitmentFeeParam"]


class CommitmentFeeParam(TypedDict, total=False):
    amount: Required[float]

    date: Required[Annotated[Union[str, datetime.date], PropertyInfo(format="iso8601")]]

    service_period_end_date: Required[
        Annotated[Union[str, datetime.datetime], PropertyInfo(alias="servicePeriodEndDate", format="iso8601")]
    ]

    service_period_start_date: Required[
        Annotated[Union[str, datetime.datetime], PropertyInfo(alias="servicePeriodStartDate", format="iso8601")]
    ]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["DebitLineItemListParams"]


class DebitLineItemListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """`nextToken` for multi page retrievals."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of line items to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TransactionTypeResponse"]


class TransactionTypeResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    archived: Optional[bool] = None
    """TRUE / FALSE flag indicating whether the data entity is archived.

    An entity can be archived if it is obsolete.
    """

    code: Optional[str] = None
    """The short code of the data entity."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this TransactionType."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO-8601 format)_ when the TransactionType was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the TransactionType was last
    modified.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """
    The unique identifier (UUID) of the user who last modified this TransactionType.
    """

    name: Optional[str] = None
    """The name of the data entity."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["JobGetOriginalDownloadURLResponse"]


class JobGetOriginalDownloadURLResponse(BaseModel):
    headers: Optional[Dict[str, str]] = None
    """The headers"""

    job_id: Optional[str] = FieldInfo(alias="jobId", default=None)
    """UUID of the download job"""

    url: Optional[str] = None
    """The URL"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["OperationalDataExportScheduleResponse"]


class OperationalDataExportScheduleResponse(BaseModel):
    id: str
    """The id of the schedule."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    operational_data_types: Optional[
        List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ]
    ] = FieldInfo(alias="operationalDataTypes", default=None)
    """A list of the entities whose operational data is included in the data export."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .job_list_params import JobListParams as JobListParams
from .schedule_list_params import ScheduleListParams as ScheduleListParams
from .schedule_create_params import ScheduleCreateParams as ScheduleCreateParams
from .schedule_list_response import ScheduleListResponse as ScheduleListResponse
from .schedule_update_params import ScheduleUpdateParams as ScheduleUpdateParams
from .destination_list_params import DestinationListParams as DestinationListParams
from .data_export_job_response import DataExportJobResponse as DataExportJobResponse
from .schedule_create_response import ScheduleCreateResponse as ScheduleCreateResponse
from .schedule_delete_response import ScheduleDeleteResponse as ScheduleDeleteResponse
from .schedule_update_response import ScheduleUpdateResponse as ScheduleUpdateResponse
from .destination_create_params import DestinationCreateParams as DestinationCreateParams
from .destination_update_params import DestinationUpdateParams as DestinationUpdateParams
from .schedule_retrieve_response import ScheduleRetrieveResponse as ScheduleRetrieveResponse
from .destination_create_response import DestinationCreateResponse as DestinationCreateResponse
from .destination_delete_response import DestinationDeleteResponse as DestinationDeleteResponse
from .destination_update_response import DestinationUpdateResponse as DestinationUpdateResponse
from .destination_retrieve_response import DestinationRetrieveResponse as DestinationRetrieveResponse
from .job_get_download_url_response import JobGetDownloadURLResponse as JobGetDownloadURLResponse
from .data_export_destination_response import DataExportDestinationResponse as DataExportDestinationResponse
from .usage_data_export_schedule_response import UsageDataExportScheduleResponse as UsageDataExportScheduleResponse
from .usage_data_export_schedule_request_param import (
    UsageDataExportScheduleRequestParam as UsageDataExportScheduleRequestParam,
)
from .operational_data_export_schedule_response import (
    OperationalDataExportScheduleResponse as OperationalDataExportScheduleResponse,
)
from .operational_data_export_schedule_request_param import (
    OperationalDataExportScheduleRequestParam as OperationalDataExportScheduleRequestParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PricingBand"]


class PricingBand(TypedDict, total=False):
    fixed_price: Required[Annotated[float, PropertyInfo(alias="fixedPrice")]]
    """Fixed price charged for the Pricing band."""

    lower_limit: Required[Annotated[float, PropertyInfo(alias="lowerLimit")]]
    """Lower limit for the Pricing band."""

    unit_price: Required[Annotated[float, PropertyInfo(alias="unitPrice")]]
    """Unit price charged for the Pricing band."""

    id: str
    """The ID for the Pricing band."""

    credit_type_id: Annotated[str, PropertyInfo(alias="creditTypeId")]
    """**OBSOLETE - this is deprecated and no longer used.**"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .permission_statement_response_param import PermissionStatementResponseParam

__all__ = ["PermissionPolicyCreateParams"]


class PermissionPolicyCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    name: Required[str]

    permission_policy: Required[
        Annotated[Iterable[PermissionStatementResponseParam], PropertyInfo(alias="permissionPolicy")]
    ]

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - do not use
      for Create. On initial Create, version is set at 1 and listed in the response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "UsageQueryParams",
    "Aggregation",
    "DimensionFilter",
    "Group",
    "GroupDataExplorerAccountGroup",
    "GroupDataExplorerDimensionGroup",
    "GroupDataExplorerTimeGroup",
]


class UsageQueryParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    end_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="endDate", format="iso8601")]]
    """ISO 8601 formatted end date to filter by."""

    start_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]]
    """ISO 8601 formatted start date to filter by."""

    account_ids: Annotated[List[str], PropertyInfo(alias="accountIds")]

    aggregations: Iterable[Aggregation]

    dimension_filters: Annotated[Iterable[DimensionFilter], PropertyInfo(alias="dimensionFilters")]

    groups: Iterable[Group]

    limit: int

    meter_ids: Annotated[List[str], PropertyInfo(alias="meterIds")]


class Aggregation(TypedDict, total=False):
    field_code: Required[Annotated[str, PropertyInfo(alias="fieldCode")]]
    """Field code"""

    field_type: Required[Annotated[Literal["DIMENSION", "MEASURE"], PropertyInfo(alias="fieldType")]]
    """Type of field"""

    function: Required[Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN", "UNIQUE"]]
    """Aggregation function"""

    meter_id: Required[Annotated[str, PropertyInfo(alias="meterId")]]
    """Meter ID"""


class DimensionFilter(TypedDict, total=False):
    field_code: Required[Annotated[str, PropertyInfo(alias="fieldCode")]]
    """Field code"""

    meter_id: Required[Annotated[str, PropertyInfo(alias="meterId")]]
    """Meter ID"""

    values: Required[List[str]]
    """Values to filter by"""


class GroupDataExplorerAccountGroup(TypedDict, total=False):
    group_type: Annotated[Literal["ACCOUNT", "DIMENSION", "TIME"], PropertyInfo(alias="groupType")]


class GroupDataExplorerDimensionGroup(TypedDict, total=False):
    field_code: Required[Annotated[str, PropertyInfo(alias="fieldCode")]]
    """Field code to group by"""

    meter_id: Required[Annotated[str, PropertyInfo(alias="meterId")]]
    """Meter ID to group by"""

    group_type: Annotated[Literal["ACCOUNT", "DIMENSION", "TIME"], PropertyInfo(alias="groupType")]


class GroupDataExplorerTimeGroup(TypedDict, total=False):
    frequency: Required[Literal["DAY", "HOUR", "WEEK", "MONTH", "QUARTER"]]
    """Frequency of usage data"""

    group_type: Annotated[Literal["ACCOUNT", "DIMENSION", "TIME"], PropertyInfo(alias="groupType")]


Group: TypeAlias = Union[GroupDataExplorerAccountGroup, GroupDataExplorerDimensionGroup, GroupDataExplorerTimeGroup]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["PlanTemplateResponse"]


class PlanTemplateResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    bill_frequency: Optional[Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY", "AD_HOC", "MIXED"]] = FieldInfo(
        alias="billFrequency", default=None
    )
    """Determines the frequency at which bills are generated.

    - **Daily**. Starting at midnight each day, covering the twenty-four hour period
      following.

    - **Weekly**. Starting at midnight on a Monday, covering the seven-day period
      following.

    - **Monthly**. Starting at midnight on the first day of each month, covering the
      entire calendar month following.

    - **Annually**. Starting at midnight on first day of each year covering the
      entire calendar year following.
    """

    bill_frequency_interval: Optional[int] = FieldInfo(alias="billFrequencyInterval", default=None)
    """How often bills are issued.

    For example, if `billFrequency` is Monthly and `billFrequencyInterval` is 3,
    bills are issued every three months.
    """

    code: Optional[str] = None
    """A unique, short code reference for the PlanTemplate.

    This code should not contain control characters or spaces.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this PlanTemplate."""

    currency: Optional[str] = None
    """
    The ISO currency code for the pricing currency used by Plans based on the Plan
    Template to define charge rates for Product consumption - for example USD, GBP,
    EUR.
    """

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO-8601 format)_ when the PlanTemplate was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the PlanTemplate was last
    modified.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) of the user who last modified this PlanTemplate."""

    minimum_spend: Optional[float] = FieldInfo(alias="minimumSpend", default=None)
    """
    The Product minimum spend amount per billing cycle for end customer Accounts on
    a pricing Plan based on the PlanTemplate. This must be a non-negative number.
    """

    minimum_spend_bill_in_advance: Optional[bool] = FieldInfo(alias="minimumSpendBillInAdvance", default=None)
    """A boolean that determines when the minimum spend is billed.

    - TRUE - minimum spend is billed at the start of each billing period.
    - FALSE - minimum spend is billed at the end of each billing period.

    Overrides the setting at Organizational level for minimum spend billing in
    arrears/in advance.
    """

    minimum_spend_description: Optional[str] = FieldInfo(alias="minimumSpendDescription", default=None)
    """Minimum spend description _(displayed on the bill line item)_."""

    name: Optional[str] = None
    """Descriptive name for the PlanTemplate."""

    ordinal: Optional[int] = None
    """The ranking of the PlanTemplate among your pricing plans.

    Lower numbers represent more basic plans, while higher numbers represent premium
    plans. This must be a non-negative integer.

    **NOTE:** **DEPRECATED** - no longer used.
    """

    product_id: Optional[str] = FieldInfo(alias="productId", default=None)
    """The unique identifier (UUID) of the Product associated with this PlanTemplate."""

    standing_charge: Optional[float] = FieldInfo(alias="standingCharge", default=None)
    """The fixed charge _(standing charge)_ applied to customer bills.

    This charge is prorated and must be a non-negative number.
    """

    standing_charge_bill_in_advance: Optional[bool] = FieldInfo(alias="standingChargeBillInAdvance", default=None)
    """A boolean that determines when the standing charge is billed.

    - TRUE - standing charge is billed at the start of each billing period.
    - FALSE - standing charge is billed at the end of each billing period.

    Overrides the setting at Organizational level for standing charge billing in
    arrears/in advance.
    """

    standing_charge_description: Optional[str] = FieldInfo(alias="standingChargeDescription", default=None)
    """Standing charge description _(displayed on the bill line item)_."""

    standing_charge_interval: Optional[int] = FieldInfo(alias="standingChargeInterval", default=None)
    """How often the standing charge is applied.

    For example, if the bill is issued every three months and
    `standingChargeInterval` is 2, then the standing charge is applied every six
    months.
    """

    standing_charge_offset: Optional[int] = FieldInfo(alias="standingChargeOffset", default=None)
    """Defines an offset for when the standing charge is first applied.

    For example, if the bill is issued every three months and the
    `standingChargeOfset` is 0, then the charge is applied to the first bill _(at
    three months)_; if 1, it would be applied to the second bill _(at six months)_,
    and so on.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import date, datetime
from typing_extensions import Literal

import httpx

from ..types import account_plan_list_params, account_plan_create_params, account_plan_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.account_plan_response import AccountPlanResponse

__all__ = ["AccountPlansResource", "AsyncAccountPlansResource"]


class AccountPlansResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountPlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AccountPlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountPlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AccountPlansResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        start_date: Union[str, datetime],
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        child_billing_mode: Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"] | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        contract_id: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        plan_group_id: str | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountPlanResponse:
        """
        Create a new AccountPlan or AccountPlanGroup.

        This endpoint creates a new AccountPlan or AccountPlanGroup for a specific
        Account in your Organization. The details of the new AccountPlan or
        AccountPlanGroup should be supplied in the request body.

        **Note:** You cannot use this call to create _both_ an AccountPlan and
        AccountPlanGroup for an Account at the same time. If you want to create both for
        an Account, you must submit two separate calls.

        Args:
          account_id: The unique identifier (UUID) for the Account.

          start_date: The start date _(in ISO-8601 format)_ for the AccountPlan or AccountPlanGroup
              becoming active for the Account.

          bill_epoch: Optional setting to define a _billing cycle date_, which acts as a reference for
              when in the applied billing frequency period bills are created:

              - For example, if you attach a Plan to an Account where the Plan is configured
                for monthly billing frequency and you've defined the period the Plan will
                apply to the Account to be from January 1st, 2022 until January 1st, 2023. You
                then set a `billEpoch` date of February 15th, 2022. The first Bill will be
                created for the Account on February 15th, and subsequent Bills created on the
                15th of the months following for the remainder of the billing period - March
                15th, April 15th, and so on.
              - If not defined, then the `billEpoch` date set for the Account will be used
                instead.
              - The date is in ISO-8601 format.

          child_billing_mode: If the Account is either a Parent or a Child Account, this specifies the Account
              hierarchy billing mode. The mode determines how billing will be handled and
              shown on bills for charges due on the Parent Account, and charges due on Child
              Accounts:

              - **Parent Breakdown** - a separate bill line item per Account. Default setting.

              - **Parent Summary** - single bill line item for all Accounts.

              - **Child** - the Child Account is billed.

          code: A unique short code for the AccountPlan or AccountPlanGroup.

          contract_id: The unique identifier (UUID) for a Contract to which you want to add the Plan or
              Plan Group being attached to the Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          end_date: The end date _(in ISO-8601 format)_ for when the AccountPlan or AccountPlanGroup
              ceases to be active for the Account. If not specified, the AccountPlan or
              AccountPlanGroup remains active indefinitely.

          plan_group_id: The unique identifier (UUID) of the PlanGroup to be attached to the Account to
              create an AccountPlanGroup.

              **Note:** Exclusive of the `planId` request parameter - exactly one of `planId`
              or `planGroupId` must be used per call.

          plan_id: The unique identifier (UUID) of the Plan to be attached to the Account to create
              an AccountPlan.

              **Note:** Exclusive of the `planGroupId` request parameter - exactly one of
              `planId` or `planGroupId` must be used per call.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/accountplans",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "start_date": start_date,
                    "bill_epoch": bill_epoch,
                    "child_billing_mode": child_billing_mode,
                    "code": code,
                    "contract_id": contract_id,
                    "custom_fields": custom_fields,
                    "end_date": end_date,
                    "plan_group_id": plan_group_id,
                    "plan_id": plan_id,
                    "version": version,
                },
                account_plan_create_params.AccountPlanCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountPlanResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountPlanResponse:
        """
        Retrieve the AccountPlan or AccountPlanGroup details corresponding to the given
        UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/accountplans/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountPlanResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        start_date: Union[str, datetime],
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        child_billing_mode: Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"] | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        contract_id: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        plan_group_id: str | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountPlanResponse:
        """
        Update the AccountPlan or AccountPlanGroup with the given UUID.

        This endpoint updates a new AccountPlan or AccountPlanGroup for a specific
        Account in your Organization. The updated information should be provided in the
        request body.

        **Notes:**

        - You cannot use this call to update _both_ an AccountPlan and AccountPlanGroup
          for an Account at the same time. If you want to update an AccounPlan and an
          AccountPlanGroup attached to an Account, you must submit two separate calls.
        - If you have created Custom Fields for an AccountPlan, when you use this
          endpoint to update the AccountPlan use the `customFields` parameter to
          preserve those Custom Fields. If you omit them from the update request, they
          will be lost.

        Args:
          account_id: The unique identifier (UUID) for the Account.

          start_date: The start date _(in ISO-8601 format)_ for the AccountPlan or AccountPlanGroup
              becoming active for the Account.

          bill_epoch: Optional setting to define a _billing cycle date_, which acts as a reference for
              when in the applied billing frequency period bills are created:

              - For example, if you attach a Plan to an Account where the Plan is configured
                for monthly billing frequency and you've defined the period the Plan will
                apply to the Account to be from January 1st, 2022 until January 1st, 2023. You
                then set a `billEpoch` date of February 15th, 2022. The first Bill will be
                created for the Account on February 15th, and subsequent Bills created on the
                15th of the months following for the remainder of the billing period - March
                15th, April 15th, and so on.
              - If not defined, then the `billEpoch` date set for the Account will be used
                instead.
              - The date is in ISO-8601 format.

          child_billing_mode: If the Account is either a Parent or a Child Account, this specifies the Account
              hierarchy billing mode. The mode determines how billing will be handled and
              shown on bills for charges due on the Parent Account, and charges due on Child
              Accounts:

              - **Parent Breakdown** - a separate bill line item per Account. Default setting.

              - **Parent Summary** - single bill line item for all Accounts.

              - **Child** - the Child Account is billed.

          code: A unique short code for the AccountPlan or AccountPlanGroup.

          contract_id: The unique identifier (UUID) for a Contract to which you want to add the Plan or
              Plan Group being attached to the Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          end_date: The end date _(in ISO-8601 format)_ for when the AccountPlan or AccountPlanGroup
              ceases to be active for the Account. If not specified, the AccountPlan or
              AccountPlanGroup remains active indefinitely.

          plan_group_id: The unique identifier (UUID) of the PlanGroup to be attached to the Account to
              create an AccountPlanGroup.

              **Note:** Exclusive of the `planId` request parameter - exactly one of `planId`
              or `planGroupId` must be used per call.

          plan_id: The unique identifier (UUID) of the Plan to be attached to the Account to create
              an AccountPlan.

              **Note:** Exclusive of the `planGroupId` request parameter - exactly one of
              `planId` or `planGroupId` must be used per call.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/accountplans/{id}",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "start_date": start_date,
                    "bill_epoch": bill_epoch,
                    "child_billing_mode": child_billing_mode,
                    "code": code,
                    "contract_id": contract_id,
                    "custom_fields": custom_fields,
                    "end_date": end_date,
                    "plan_group_id": plan_group_id,
                    "plan_id": plan_id,
                    "version": version,
                },
                account_plan_update_params.AccountPlanUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountPlanResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account: str | NotGiven = NOT_GIVEN,
        contract: Optional[str] | NotGiven = NOT_GIVEN,
        date: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        includeall: bool | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        plan: str | NotGiven = NOT_GIVEN,
        product: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[AccountPlanResponse]:
        """
        Retrieve a list of AccountPlan and AccountPlanGroup entities for the specified
        Organization.

        This endpoint retrieves a list of AccountPlans and AccountPlanGroups for a
        specific Organization. The list can be paginated for easier management, and
        supports filtering with various parameters.

        **NOTE:** You cannot use the `product` query parameter as a single filter
        condition, but must always use it in combination with the `account` query
        parameter.

        Args:
          account: The unique identifier (UUID) for the Account whose AccountPlans and
              AccountPlanGroups you want to retrieve.

          date: The specific date for which you want to retrieve active AccountPlans and
              AccountPlanGroups.

          ids: A list of unique identifiers (UUIDs) for specific AccountPlans and
              AccountPlanGroups you want to retrieve.

          includeall: A Boolean flag that specifies whether to include both active and inactive
              AccountPlans and AccountPlanGroups in the list.

              - **TRUE** - both active and inactive AccountPlans and AccountPlanGroups are
                included in the list.
              - **FALSE** - only active AccountPlans and AccountPlanGroups are retrieved in
                the list.

          next_token: The `nextToken` for retrieving the next page of AccountPlans and
              AccountPlanGroups. It is used to fetch the next page of AccountPlans and
              AccountPlanGroups in a paginated list.

          page_size: The maximum number of AccountPlans and AccountPlanGroups to return per page.

          plan: The unique identifier (UUID) for the Plan or Plan Group whose associated
              AccountPlans or AccountPlanGroups you want to retrieve.

          product: The unique identifier (UUID) for the Product whose associated AccountPlans you
              want to retrieve.

              **NOTE:** You cannot use the `product` query parameter as a single filter
              condition, but must always use it in combination with the `account` query
              parameter.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/accountplans",
            page=SyncCursor[AccountPlanResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account": account,
                        "contract": contract,
                        "date": date,
                        "ids": ids,
                        "includeall": includeall,
                        "next_token": next_token,
                        "page_size": page_size,
                        "plan": plan,
                        "product": product,
                    },
                    account_plan_list_params.AccountPlanListParams,
                ),
            ),
            model=AccountPlanResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountPlanResponse:
        """
        Delete the AccountPlan or AccountPlanGroup with the given UUID.

        This endpoint deletes an AccountPlan or AccountPlanGroup that has been attached
        to a specific Account in your Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/accountplans/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountPlanResponse,
        )


class AsyncAccountPlansResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountPlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountPlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountPlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncAccountPlansResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        start_date: Union[str, datetime],
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        child_billing_mode: Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"] | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        contract_id: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        plan_group_id: str | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountPlanResponse:
        """
        Create a new AccountPlan or AccountPlanGroup.

        This endpoint creates a new AccountPlan or AccountPlanGroup for a specific
        Account in your Organization. The details of the new AccountPlan or
        AccountPlanGroup should be supplied in the request body.

        **Note:** You cannot use this call to create _both_ an AccountPlan and
        AccountPlanGroup for an Account at the same time. If you want to create both for
        an Account, you must submit two separate calls.

        Args:
          account_id: The unique identifier (UUID) for the Account.

          start_date: The start date _(in ISO-8601 format)_ for the AccountPlan or AccountPlanGroup
              becoming active for the Account.

          bill_epoch: Optional setting to define a _billing cycle date_, which acts as a reference for
              when in the applied billing frequency period bills are created:

              - For example, if you attach a Plan to an Account where the Plan is configured
                for monthly billing frequency and you've defined the period the Plan will
                apply to the Account to be from January 1st, 2022 until January 1st, 2023. You
                then set a `billEpoch` date of February 15th, 2022. The first Bill will be
                created for the Account on February 15th, and subsequent Bills created on the
                15th of the months following for the remainder of the billing period - March
                15th, April 15th, and so on.
              - If not defined, then the `billEpoch` date set for the Account will be used
                instead.
              - The date is in ISO-8601 format.

          child_billing_mode: If the Account is either a Parent or a Child Account, this specifies the Account
              hierarchy billing mode. The mode determines how billing will be handled and
              shown on bills for charges due on the Parent Account, and charges due on Child
              Accounts:

              - **Parent Breakdown** - a separate bill line item per Account. Default setting.

              - **Parent Summary** - single bill line item for all Accounts.

              - **Child** - the Child Account is billed.

          code: A unique short code for the AccountPlan or AccountPlanGroup.

          contract_id: The unique identifier (UUID) for a Contract to which you want to add the Plan or
              Plan Group being attached to the Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          end_date: The end date _(in ISO-8601 format)_ for when the AccountPlan or AccountPlanGroup
              ceases to be active for the Account. If not specified, the AccountPlan or
              AccountPlanGroup remains active indefinitely.

          plan_group_id: The unique identifier (UUID) of the PlanGroup to be attached to the Account to
              create an AccountPlanGroup.

              **Note:** Exclusive of the `planId` request parameter - exactly one of `planId`
              or `planGroupId` must be used per call.

          plan_id: The unique identifier (UUID) of the Plan to be attached to the Account to create
              an AccountPlan.

              **Note:** Exclusive of the `planGroupId` request parameter - exactly one of
              `planId` or `planGroupId` must be used per call.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/accountplans",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "start_date": start_date,
                    "bill_epoch": bill_epoch,
                    "child_billing_mode": child_billing_mode,
                    "code": code,
                    "contract_id": contract_id,
                    "custom_fields": custom_fields,
                    "end_date": end_date,
                    "plan_group_id": plan_group_id,
                    "plan_id": plan_id,
                    "version": version,
                },
                account_plan_create_params.AccountPlanCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountPlanResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountPlanResponse:
        """
        Retrieve the AccountPlan or AccountPlanGroup details corresponding to the given
        UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/accountplans/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountPlanResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        start_date: Union[str, datetime],
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        child_billing_mode: Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"] | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        contract_id: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        plan_group_id: str | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountPlanResponse:
        """
        Update the AccountPlan or AccountPlanGroup with the given UUID.

        This endpoint updates a new AccountPlan or AccountPlanGroup for a specific
        Account in your Organization. The updated information should be provided in the
        request body.

        **Notes:**

        - You cannot use this call to update _both_ an AccountPlan and AccountPlanGroup
          for an Account at the same time. If you want to update an AccounPlan and an
          AccountPlanGroup attached to an Account, you must submit two separate calls.
        - If you have created Custom Fields for an AccountPlan, when you use this
          endpoint to update the AccountPlan use the `customFields` parameter to
          preserve those Custom Fields. If you omit them from the update request, they
          will be lost.

        Args:
          account_id: The unique identifier (UUID) for the Account.

          start_date: The start date _(in ISO-8601 format)_ for the AccountPlan or AccountPlanGroup
              becoming active for the Account.

          bill_epoch: Optional setting to define a _billing cycle date_, which acts as a reference for
              when in the applied billing frequency period bills are created:

              - For example, if you attach a Plan to an Account where the Plan is configured
                for monthly billing frequency and you've defined the period the Plan will
                apply to the Account to be from January 1st, 2022 until January 1st, 2023. You
                then set a `billEpoch` date of February 15th, 2022. The first Bill will be
                created for the Account on February 15th, and subsequent Bills created on the
                15th of the months following for the remainder of the billing period - March
                15th, April 15th, and so on.
              - If not defined, then the `billEpoch` date set for the Account will be used
                instead.
              - The date is in ISO-8601 format.

          child_billing_mode: If the Account is either a Parent or a Child Account, this specifies the Account
              hierarchy billing mode. The mode determines how billing will be handled and
              shown on bills for charges due on the Parent Account, and charges due on Child
              Accounts:

              - **Parent Breakdown** - a separate bill line item per Account. Default setting.

              - **Parent Summary** - single bill line item for all Accounts.

              - **Child** - the Child Account is billed.

          code: A unique short code for the AccountPlan or AccountPlanGroup.

          contract_id: The unique identifier (UUID) for a Contract to which you want to add the Plan or
              Plan Group being attached to the Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          end_date: The end date _(in ISO-8601 format)_ for when the AccountPlan or AccountPlanGroup
              ceases to be active for the Account. If not specified, the AccountPlan or
              AccountPlanGroup remains active indefinitely.

          plan_group_id: The unique identifier (UUID) of the PlanGroup to be attached to the Account to
              create an AccountPlanGroup.

              **Note:** Exclusive of the `planId` request parameter - exactly one of `planId`
              or `planGroupId` must be used per call.

          plan_id: The unique identifier (UUID) of the Plan to be attached to the Account to create
              an AccountPlan.

              **Note:** Exclusive of the `planGroupId` request parameter - exactly one of
              `planId` or `planGroupId` must be used per call.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/accountplans/{id}",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "start_date": start_date,
                    "bill_epoch": bill_epoch,
                    "child_billing_mode": child_billing_mode,
                    "code": code,
                    "contract_id": contract_id,
                    "custom_fields": custom_fields,
                    "end_date": end_date,
                    "plan_group_id": plan_group_id,
                    "plan_id": plan_id,
                    "version": version,
                },
                account_plan_update_params.AccountPlanUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountPlanResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account: str | NotGiven = NOT_GIVEN,
        contract: Optional[str] | NotGiven = NOT_GIVEN,
        date: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        includeall: bool | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        plan: str | NotGiven = NOT_GIVEN,
        product: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[AccountPlanResponse, AsyncCursor[AccountPlanResponse]]:
        """
        Retrieve a list of AccountPlan and AccountPlanGroup entities for the specified
        Organization.

        This endpoint retrieves a list of AccountPlans and AccountPlanGroups for a
        specific Organization. The list can be paginated for easier management, and
        supports filtering with various parameters.

        **NOTE:** You cannot use the `product` query parameter as a single filter
        condition, but must always use it in combination with the `account` query
        parameter.

        Args:
          account: The unique identifier (UUID) for the Account whose AccountPlans and
              AccountPlanGroups you want to retrieve.

          date: The specific date for which you want to retrieve active AccountPlans and
              AccountPlanGroups.

          ids: A list of unique identifiers (UUIDs) for specific AccountPlans and
              AccountPlanGroups you want to retrieve.

          includeall: A Boolean flag that specifies whether to include both active and inactive
              AccountPlans and AccountPlanGroups in the list.

              - **TRUE** - both active and inactive AccountPlans and AccountPlanGroups are
                included in the list.
              - **FALSE** - only active AccountPlans and AccountPlanGroups are retrieved in
                the list.

          next_token: The `nextToken` for retrieving the next page of AccountPlans and
              AccountPlanGroups. It is used to fetch the next page of AccountPlans and
              AccountPlanGroups in a paginated list.

          page_size: The maximum number of AccountPlans and AccountPlanGroups to return per page.

          plan: The unique identifier (UUID) for the Plan or Plan Group whose associated
              AccountPlans or AccountPlanGroups you want to retrieve.

          product: The unique identifier (UUID) for the Product whose associated AccountPlans you
              want to retrieve.

              **NOTE:** You cannot use the `product` query parameter as a single filter
              condition, but must always use it in combination with the `account` query
              parameter.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/accountplans",
            page=AsyncCursor[AccountPlanResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account": account,
                        "contract": contract,
                        "date": date,
                        "ids": ids,
                        "includeall": includeall,
                        "next_token": next_token,
                        "page_size": page_size,
                        "plan": plan,
                        "product": product,
                    },
                    account_plan_list_params.AccountPlanListParams,
                ),
            ),
            model=AccountPlanResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountPlanResponse:
        """
        Delete the AccountPlan or AccountPlanGroup with the given UUID.

        This endpoint deletes an AccountPlan or AccountPlanGroup that has been attached
        to a specific Account in your Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/accountplans/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountPlanResponse,
        )


class AccountPlansResourceWithRawResponse:
    def __init__(self, account_plans: AccountPlansResource) -> None:
        self._account_plans = account_plans

        self.create = to_raw_response_wrapper(
            account_plans.create,
        )
        self.retrieve = to_raw_response_wrapper(
            account_plans.retrieve,
        )
        self.update = to_raw_response_wrapper(
            account_plans.update,
        )
        self.list = to_raw_response_wrapper(
            account_plans.list,
        )
        self.delete = to_raw_response_wrapper(
            account_plans.delete,
        )


class AsyncAccountPlansResourceWithRawResponse:
    def __init__(self, account_plans: AsyncAccountPlansResource) -> None:
        self._account_plans = account_plans

        self.create = async_to_raw_response_wrapper(
            account_plans.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            account_plans.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            account_plans.update,
        )
        self.list = async_to_raw_response_wrapper(
            account_plans.list,
        )
        self.delete = async_to_raw_response_wrapper(
            account_plans.delete,
        )


class AccountPlansResourceWithStreamingResponse:
    def __init__(self, account_plans: AccountPlansResource) -> None:
        self._account_plans = account_plans

        self.create = to_streamed_response_wrapper(
            account_plans.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            account_plans.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            account_plans.update,
        )
        self.list = to_streamed_response_wrapper(
            account_plans.list,
        )
        self.delete = to_streamed_response_wrapper(
            account_plans.delete,
        )


class AsyncAccountPlansResourceWithStreamingResponse:
    def __init__(self, account_plans: AsyncAccountPlansResource) -> None:
        self._account_plans = account_plans

        self.create = async_to_streamed_response_wrapper(
            account_plans.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            account_plans.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            account_plans.update,
        )
        self.list = async_to_streamed_response_wrapper(
            account_plans.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            account_plans.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AddressParam"]


class AddressParam(TypedDict, total=False):
    address_line1: Annotated[str, PropertyInfo(alias="addressLine1")]

    address_line2: Annotated[str, PropertyInfo(alias="addressLine2")]

    address_line3: Annotated[str, PropertyInfo(alias="addressLine3")]

    address_line4: Annotated[str, PropertyInfo(alias="addressLine4")]

    country: str

    locality: str

    post_code: Annotated[str, PropertyInfo(alias="postCode")]

    region: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import (
    ContractResponse,
    ContractEndDateBillingEntitiesResponse,
)
from tests.utils import assert_matches_type
from m3ter._utils import parse_date, parse_datetime
from m3ter.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestContracts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        contract = client.contracts.create(
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        contract = client.contracts.create(
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
            code="JS!?Q0]r] ]$]",
            custom_fields={"foo": "string"},
            description="description",
            purchase_order_number="purchaseOrderNumber",
            version=0,
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.contracts.with_raw_response.create(
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.contracts.with_streaming_response.create(
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.contracts.with_raw_response.create(
                org_id="",
                account_id="x",
                end_date=parse_date("2019-12-27"),
                name="x",
                start_date=parse_date("2019-12-27"),
            )

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        contract = client.contracts.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.contracts.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.contracts.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.contracts.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.contracts.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        contract = client.contracts.update(
            id="id",
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        contract = client.contracts.update(
            id="id",
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
            code="JS!?Q0]r] ]$]",
            custom_fields={"foo": "string"},
            description="description",
            purchase_order_number="purchaseOrderNumber",
            version=0,
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.contracts.with_raw_response.update(
            id="id",
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.contracts.with_streaming_response.update(
            id="id",
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.contracts.with_raw_response.update(
                id="id",
                org_id="",
                account_id="x",
                end_date=parse_date("2019-12-27"),
                name="x",
                start_date=parse_date("2019-12-27"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.contracts.with_raw_response.update(
                id="",
                org_id="orgId",
                account_id="x",
                end_date=parse_date("2019-12-27"),
                name="x",
                start_date=parse_date("2019-12-27"),
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        contract = client.contracts.list(
            org_id="orgId",
        )
        assert_matches_type(SyncCursor[ContractResponse], contract, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        contract = client.contracts.list(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            codes=["string"],
            ids=["string"],
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(SyncCursor[ContractResponse], contract, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.contracts.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(SyncCursor[ContractResponse], contract, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.contracts.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(SyncCursor[ContractResponse], contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.contracts.with_raw_response.list(
                org_id="",
            )

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        contract = client.contracts.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.contracts.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.contracts.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.contracts.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.contracts.with_raw_response.delete(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_end_date_billing_entities(self, client: M3ter) -> None:
        contract = client.contracts.end_date_billing_entities(
            id="id",
            org_id="orgId",
            billing_entities=["CONTRACT"],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ContractEndDateBillingEntitiesResponse, contract, path=["response"])

    @parametrize
    def test_method_end_date_billing_entities_with_all_params(self, client: M3ter) -> None:
        contract = client.contracts.end_date_billing_entities(
            id="id",
            org_id="orgId",
            billing_entities=["CONTRACT"],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            apply_to_children=True,
        )
        assert_matches_type(ContractEndDateBillingEntitiesResponse, contract, path=["response"])

    @parametrize
    def test_raw_response_end_date_billing_entities(self, client: M3ter) -> None:
        response = client.contracts.with_raw_response.end_date_billing_entities(
            id="id",
            org_id="orgId",
            billing_entities=["CONTRACT"],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = response.parse()
        assert_matches_type(ContractEndDateBillingEntitiesResponse, contract, path=["response"])

    @parametrize
    def test_streaming_response_end_date_billing_entities(self, client: M3ter) -> None:
        with client.contracts.with_streaming_response.end_date_billing_entities(
            id="id",
            org_id="orgId",
            billing_entities=["CONTRACT"],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = response.parse()
            assert_matches_type(ContractEndDateBillingEntitiesResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_end_date_billing_entities(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.contracts.with_raw_response.end_date_billing_entities(
                id="id",
                org_id="",
                billing_entities=["CONTRACT"],
                end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.contracts.with_raw_response.end_date_billing_entities(
                id="",
                org_id="orgId",
                billing_entities=["CONTRACT"],
                end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )


class TestAsyncContracts:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.create(
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.create(
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
            code="JS!?Q0]r] ]$]",
            custom_fields={"foo": "string"},
            description="description",
            purchase_order_number="purchaseOrderNumber",
            version=0,
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.contracts.with_raw_response.create(
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.contracts.with_streaming_response.create(
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.contracts.with_raw_response.create(
                org_id="",
                account_id="x",
                end_date=parse_date("2019-12-27"),
                name="x",
                start_date=parse_date("2019-12-27"),
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.contracts.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.contracts.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.contracts.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.contracts.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.update(
            id="id",
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.update(
            id="id",
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
            code="JS!?Q0]r] ]$]",
            custom_fields={"foo": "string"},
            description="description",
            purchase_order_number="purchaseOrderNumber",
            version=0,
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.contracts.with_raw_response.update(
            id="id",
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.contracts.with_streaming_response.update(
            id="id",
            org_id="orgId",
            account_id="x",
            end_date=parse_date("2019-12-27"),
            name="x",
            start_date=parse_date("2019-12-27"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.contracts.with_raw_response.update(
                id="id",
                org_id="",
                account_id="x",
                end_date=parse_date("2019-12-27"),
                name="x",
                start_date=parse_date("2019-12-27"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.contracts.with_raw_response.update(
                id="",
                org_id="orgId",
                account_id="x",
                end_date=parse_date("2019-12-27"),
                name="x",
                start_date=parse_date("2019-12-27"),
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.list(
            org_id="orgId",
        )
        assert_matches_type(AsyncCursor[ContractResponse], contract, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.list(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            codes=["string"],
            ids=["string"],
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(AsyncCursor[ContractResponse], contract, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.contracts.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(AsyncCursor[ContractResponse], contract, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.contracts.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(AsyncCursor[ContractResponse], contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.contracts.with_raw_response.list(
                org_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.contracts.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.contracts.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.contracts.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.contracts.with_raw_response.delete(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_end_date_billing_entities(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.end_date_billing_entities(
            id="id",
            org_id="orgId",
            billing_entities=["CONTRACT"],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ContractEndDateBillingEntitiesResponse, contract, path=["response"])

    @parametrize
    async def test_method_end_date_billing_entities_with_all_params(self, async_client: AsyncM3ter) -> None:
        contract = await async_client.contracts.end_date_billing_entities(
            id="id",
            org_id="orgId",
            billing_entities=["CONTRACT"],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            apply_to_children=True,
        )
        assert_matches_type(ContractEndDateBillingEntitiesResponse, contract, path=["response"])

    @parametrize
    async def test_raw_response_end_date_billing_entities(self, async_client: AsyncM3ter) -> None:
        response = await async_client.contracts.with_raw_response.end_date_billing_entities(
            id="id",
            org_id="orgId",
            billing_entities=["CONTRACT"],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contract = await response.parse()
        assert_matches_type(ContractEndDateBillingEntitiesResponse, contract, path=["response"])

    @parametrize
    async def test_streaming_response_end_date_billing_entities(self, async_client: AsyncM3ter) -> None:
        async with async_client.contracts.with_streaming_response.end_date_billing_entities(
            id="id",
            org_id="orgId",
            billing_entities=["CONTRACT"],
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contract = await response.parse()
            assert_matches_type(ContractEndDateBillingEntitiesResponse, contract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_end_date_billing_entities(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.contracts.with_raw_response.end_date_billing_entities(
                id="id",
                org_id="",
                billing_entities=["CONTRACT"],
                end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.contracts.with_raw_response.end_date_billing_entities(
                id="",
                org_id="orgId",
                billing_entities=["CONTRACT"],
                end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["FileUploadJobResponse"]


class FileUploadJobResponse(BaseModel):
    id: Optional[str] = None
    """UUID of the file upload job."""

    content_length: Optional[int] = FieldInfo(alias="contentLength", default=None)
    """The size of the body in bytes.

    For example: `"contentLength": 485`, where 485 is the size in bytes of the file
    uploaded.
    """

    failed_rows: Optional[int] = FieldInfo(alias="failedRows", default=None)
    """The number of rows that failed processing during ingest."""

    file_name: Optional[str] = FieldInfo(alias="fileName", default=None)
    """The name of the measurements file for the upload job."""

    processed_rows: Optional[int] = FieldInfo(alias="processedRows", default=None)
    """The number of rows that were processed during ingest."""

    status: Optional[Literal["notUploaded", "running", "failed", "succeeded"]] = None
    """The status of the file upload job."""

    total_rows: Optional[int] = FieldInfo(alias="totalRows", default=None)
    """The total number of rows in the file."""

    upload_date: Optional[str] = FieldInfo(alias="uploadDate", default=None)
    """The upload date for the upload job _(in ISO-8601 format)_."""

    version: Optional[int] = None
    """The version number. Default value when newly created is one."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["BillApproveResponse"]


class BillApproveResponse(BaseModel):
    message: Optional[str] = None
    """
    A message indicating the success or failure of the Bills' approval, along with
    relevant details.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["DataExportJobResponse"]


class DataExportJobResponse(BaseModel):
    id: str
    """The id of the Export Job."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    date_created: Optional[datetime] = FieldInfo(alias="dateCreated", default=None)
    """When the data Export Job was created."""

    schedule_id: Optional[str] = FieldInfo(alias="scheduleId", default=None)
    """The id of the data Export Schedule."""

    source_type: Optional[Literal["USAGE", "OPERATIONAL"]] = FieldInfo(alias="sourceType", default=None)

    started_at: Optional[datetime] = FieldInfo(alias="startedAt", default=None)
    """When the data Export Job started running"""

    status: Optional[Literal["PENDING", "RUNNING", "SUCCEEDED", "FAILED"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import (
    BillResponse,
    BillSearchResponse,
    BillApproveResponse,
)
from tests.utils import assert_matches_type
from m3ter.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBills:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        bill = client.bills.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.bills.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.bills.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bills.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.bills.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        bill = client.bills.list(
            org_id="orgId",
        )
        assert_matches_type(SyncCursor[BillResponse], bill, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        bill = client.bills.list(
            org_id="orgId",
            account_id="accountId",
            bill_date="billDate",
            bill_date_end="billDateEnd",
            bill_date_start="billDateStart",
            billing_frequency="billingFrequency",
            exclude_line_items=True,
            external_invoice_date_end="externalInvoiceDateEnd",
            external_invoice_date_start="externalInvoiceDateStart",
            ids=["string"],
            include_bill_total=True,
            locked=True,
            next_token="nextToken",
            page_size=1,
            status="PENDING",
        )
        assert_matches_type(SyncCursor[BillResponse], bill, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.bills.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = response.parse()
        assert_matches_type(SyncCursor[BillResponse], bill, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.bills.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = response.parse()
            assert_matches_type(SyncCursor[BillResponse], bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bills.with_raw_response.list(
                org_id="",
            )

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        bill = client.bills.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.bills.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.bills.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bills.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.bills.with_raw_response.delete(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_approve(self, client: M3ter) -> None:
        bill = client.bills.approve(
            org_id="orgId",
            bill_ids=["string"],
        )
        assert_matches_type(BillApproveResponse, bill, path=["response"])

    @parametrize
    def test_method_approve_with_all_params(self, client: M3ter) -> None:
        bill = client.bills.approve(
            org_id="orgId",
            bill_ids=["string"],
            account_ids="accountIds",
            external_invoice_date_end="externalInvoiceDateEnd",
            external_invoice_date_start="externalInvoiceDateStart",
        )
        assert_matches_type(BillApproveResponse, bill, path=["response"])

    @parametrize
    def test_raw_response_approve(self, client: M3ter) -> None:
        response = client.bills.with_raw_response.approve(
            org_id="orgId",
            bill_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = response.parse()
        assert_matches_type(BillApproveResponse, bill, path=["response"])

    @parametrize
    def test_streaming_response_approve(self, client: M3ter) -> None:
        with client.bills.with_streaming_response.approve(
            org_id="orgId",
            bill_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = response.parse()
            assert_matches_type(BillApproveResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_approve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bills.with_raw_response.approve(
                org_id="",
                bill_ids=["string"],
            )

    @parametrize
    def test_method_latest_by_account(self, client: M3ter) -> None:
        bill = client.bills.latest_by_account(
            account_id="accountId",
            org_id="orgId",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_raw_response_latest_by_account(self, client: M3ter) -> None:
        response = client.bills.with_raw_response.latest_by_account(
            account_id="accountId",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_streaming_response_latest_by_account(self, client: M3ter) -> None:
        with client.bills.with_streaming_response.latest_by_account(
            account_id="accountId",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_latest_by_account(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bills.with_raw_response.latest_by_account(
                account_id="accountId",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.bills.with_raw_response.latest_by_account(
                account_id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_lock(self, client: M3ter) -> None:
        bill = client.bills.lock(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_raw_response_lock(self, client: M3ter) -> None:
        response = client.bills.with_raw_response.lock(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_streaming_response_lock(self, client: M3ter) -> None:
        with client.bills.with_streaming_response.lock(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_lock(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bills.with_raw_response.lock(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.bills.with_raw_response.lock(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_search(self, client: M3ter) -> None:
        bill = client.bills.search(
            org_id="orgId",
        )
        assert_matches_type(BillSearchResponse, bill, path=["response"])

    @parametrize
    def test_method_search_with_all_params(self, client: M3ter) -> None:
        bill = client.bills.search(
            org_id="orgId",
            from_document=0,
            operator="AND",
            page_size=1,
            search_query="searchQuery",
            sort_by="sortBy",
            sort_order="ASC",
        )
        assert_matches_type(BillSearchResponse, bill, path=["response"])

    @parametrize
    def test_raw_response_search(self, client: M3ter) -> None:
        response = client.bills.with_raw_response.search(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = response.parse()
        assert_matches_type(BillSearchResponse, bill, path=["response"])

    @parametrize
    def test_streaming_response_search(self, client: M3ter) -> None:
        with client.bills.with_streaming_response.search(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = response.parse()
            assert_matches_type(BillSearchResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_search(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bills.with_raw_response.search(
                org_id="",
            )

    @parametrize
    def test_method_update_status(self, client: M3ter) -> None:
        bill = client.bills.update_status(
            id="id",
            org_id="orgId",
            status="PENDING",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_raw_response_update_status(self, client: M3ter) -> None:
        response = client.bills.with_raw_response.update_status(
            id="id",
            org_id="orgId",
            status="PENDING",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    def test_streaming_response_update_status(self, client: M3ter) -> None:
        with client.bills.with_streaming_response.update_status(
            id="id",
            org_id="orgId",
            status="PENDING",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_status(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bills.with_raw_response.update_status(
                id="id",
                org_id="",
                status="PENDING",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.bills.with_raw_response.update_status(
                id="",
                org_id="orgId",
                status="PENDING",
            )


class TestAsyncBills:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bills.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = await response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.bills.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = await response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bills.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.bills.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.list(
            org_id="orgId",
        )
        assert_matches_type(AsyncCursor[BillResponse], bill, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.list(
            org_id="orgId",
            account_id="accountId",
            bill_date="billDate",
            bill_date_end="billDateEnd",
            bill_date_start="billDateStart",
            billing_frequency="billingFrequency",
            exclude_line_items=True,
            external_invoice_date_end="externalInvoiceDateEnd",
            external_invoice_date_start="externalInvoiceDateStart",
            ids=["string"],
            include_bill_total=True,
            locked=True,
            next_token="nextToken",
            page_size=1,
            status="PENDING",
        )
        assert_matches_type(AsyncCursor[BillResponse], bill, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bills.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = await response.parse()
        assert_matches_type(AsyncCursor[BillResponse], bill, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.bills.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = await response.parse()
            assert_matches_type(AsyncCursor[BillResponse], bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bills.with_raw_response.list(
                org_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bills.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = await response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.bills.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = await response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bills.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.bills.with_raw_response.delete(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_approve(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.approve(
            org_id="orgId",
            bill_ids=["string"],
        )
        assert_matches_type(BillApproveResponse, bill, path=["response"])

    @parametrize
    async def test_method_approve_with_all_params(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.approve(
            org_id="orgId",
            bill_ids=["string"],
            account_ids="accountIds",
            external_invoice_date_end="externalInvoiceDateEnd",
            external_invoice_date_start="externalInvoiceDateStart",
        )
        assert_matches_type(BillApproveResponse, bill, path=["response"])

    @parametrize
    async def test_raw_response_approve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bills.with_raw_response.approve(
            org_id="orgId",
            bill_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = await response.parse()
        assert_matches_type(BillApproveResponse, bill, path=["response"])

    @parametrize
    async def test_streaming_response_approve(self, async_client: AsyncM3ter) -> None:
        async with async_client.bills.with_streaming_response.approve(
            org_id="orgId",
            bill_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = await response.parse()
            assert_matches_type(BillApproveResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_approve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bills.with_raw_response.approve(
                org_id="",
                bill_ids=["string"],
            )

    @parametrize
    async def test_method_latest_by_account(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.latest_by_account(
            account_id="accountId",
            org_id="orgId",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_raw_response_latest_by_account(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bills.with_raw_response.latest_by_account(
            account_id="accountId",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = await response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_streaming_response_latest_by_account(self, async_client: AsyncM3ter) -> None:
        async with async_client.bills.with_streaming_response.latest_by_account(
            account_id="accountId",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = await response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_latest_by_account(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bills.with_raw_response.latest_by_account(
                account_id="accountId",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.bills.with_raw_response.latest_by_account(
                account_id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_lock(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.lock(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_raw_response_lock(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bills.with_raw_response.lock(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = await response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_streaming_response_lock(self, async_client: AsyncM3ter) -> None:
        async with async_client.bills.with_streaming_response.lock(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = await response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_lock(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bills.with_raw_response.lock(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.bills.with_raw_response.lock(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_search(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.search(
            org_id="orgId",
        )
        assert_matches_type(BillSearchResponse, bill, path=["response"])

    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.search(
            org_id="orgId",
            from_document=0,
            operator="AND",
            page_size=1,
            search_query="searchQuery",
            sort_by="sortBy",
            sort_order="ASC",
        )
        assert_matches_type(BillSearchResponse, bill, path=["response"])

    @parametrize
    async def test_raw_response_search(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bills.with_raw_response.search(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = await response.parse()
        assert_matches_type(BillSearchResponse, bill, path=["response"])

    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncM3ter) -> None:
        async with async_client.bills.with_streaming_response.search(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = await response.parse()
            assert_matches_type(BillSearchResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_search(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bills.with_raw_response.search(
                org_id="",
            )

    @parametrize
    async def test_method_update_status(self, async_client: AsyncM3ter) -> None:
        bill = await async_client.bills.update_status(
            id="id",
            org_id="orgId",
            status="PENDING",
        )
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_raw_response_update_status(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bills.with_raw_response.update_status(
            id="id",
            org_id="orgId",
            status="PENDING",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill = await response.parse()
        assert_matches_type(BillResponse, bill, path=["response"])

    @parametrize
    async def test_streaming_response_update_status(self, async_client: AsyncM3ter) -> None:
        async with async_client.bills.with_streaming_response.update_status(
            id="id",
            org_id="orgId",
            status="PENDING",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill = await response.parse()
            assert_matches_type(BillResponse, bill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_status(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bills.with_raw_response.update_status(
                id="id",
                org_id="",
                status="PENDING",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.bills.with_raw_response.update_status(
                id="",
                org_id="orgId",
                status="PENDING",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TransactionTypeListParams"]


class TransactionTypeListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    archived: bool
    """
    Filter with this Boolean flag whether to include TransactionTypes that are
    archived.

    - TRUE - include archived TransactionTypes in the list.
    - FALSE - exclude archived TransactionTypes.
    """

    codes: List[str]
    """A list of TransactionType short codes to retrieve."""

    ids: List[str]
    """A list of TransactionType unique identifiers (UUIDs) to retrieve."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of TransactionTypes in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of TransactionTypes to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["UsageQueryResponse"]


class UsageQueryResponse(BaseModel):
    data: Optional[List[Dict[str, object]]] = None

    has_more_data: Optional[bool] = FieldInfo(alias="hasMoreData", default=None)
    """Flag to know if there are more data available than the one returned"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["EventGetFieldsParams"]


class EventGetFieldsParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    event_name: Annotated[str, PropertyInfo(alias="eventName")]
    """
    The name of the specific Event Type to use as a list filter, for example
    `configuration.commitment.created`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .permission_statement_response import PermissionStatementResponse

__all__ = ["UserResponse"]


class UserResponse(BaseModel):
    id: Optional[str] = None
    """The unique identifier (UUID) of this user."""

    contact_number: Optional[str] = FieldInfo(alias="contactNumber", default=None)
    """The user's contact telephone number."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The user who created this user."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO-8601 format)_ when the user was created."""

    dt_end_access: Optional[datetime] = FieldInfo(alias="dtEndAccess", default=None)
    """The date and time _(in ISO 8601 format)_ when the user's access will end.

    Used to set or update the date and time a user's access expires.
    """

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The date and time _(in ISO-8601 format)_ when the user was last modified."""

    email: Optional[str] = None
    """The email address for this user."""

    first_accepted_terms_and_conditions: Optional[datetime] = FieldInfo(
        alias="firstAcceptedTermsAndConditions", default=None
    )
    """
    The date and time _(in ISO 8601 format)_ when this user first accepted the the
    m3ter terms and conditions.
    """

    first_name: Optional[str] = FieldInfo(alias="firstName", default=None)
    """The first name of the user."""

    last_accepted_terms_and_conditions: Optional[datetime] = FieldInfo(
        alias="lastAcceptedTermsAndConditions", default=None
    )
    """
    The date and time _(in ISO 8601 format)_ when this user last accepted the the
    m3ter terms and conditions.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) of the user who last modified this user record."""

    last_name: Optional[str] = FieldInfo(alias="lastName", default=None)
    """The surname of the user."""

    organizations: Optional[List[str]] = None
    """An array listing the Organizations where this user has access."""

    permission_policy: Optional[List[PermissionStatementResponse]] = FieldInfo(alias="permissionPolicy", default=None)
    """An array of permission statements for the user.

    Each permission statement defines a specific permission for the user.
    """

    support_user: Optional[bool] = FieldInfo(alias="supportUser", default=None)
    """Indicates whether this is a m3ter Support user."""

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """
